#ifndef __GNAPSTER_H
#define __GNAPSTER_H

#include "config.h"
#ifdef HAVE_GNOME
#include <gnome.h>
#else
#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <libintl.h>
#define _(string) dgettext(PACKAGE, string)
#define N_(string) string
#endif /* HAVE_GNOME */

#include "callbacks.h"
#include "fuckgnome.h"
#include <errno.h>

#define NAP_VERSION "Gnapster 1.4.1a"

/* default napster port */
#define NAP_PORT 6699

#define NA_RESET_CONTINUE() { na_err = 0; continue; }
#define NA_RESET_BREAK() { na_err = 0; break; }
#define NA_RESET_RETURN(x) { na_err = 0; return (x); }
#define NA_RESET() na_err = 0

#define NA_ERR_HANDLE(x) if (na_err) NA_RESET_RETURN(x)
#define NA_ERR_HANDLE_CONTINUE() if (na_err) NA_RESET_CONTINUE()
#define NA_ERR_HANDLE_BREAK() if (na_err) NA_RESET_BREAK()

#define INT(x) GPOINTER_TO_INT(x)

/* these are in place to make sure we are not closing/removing
 * things twice */
#define CLOSE(x) if (x >= 0) { close(x); }
#define INPUT_REMOVE(x) if (x > 0) { gdk_input_remove(x); }

#define MESSAGE ""
#define SYSTEM "system"
#define SYSTEM_N "system_negative"
#define SYSTEM_P "system_positive"

#define MAX_HISTORY 15

#define DOWNLOAD_FORCE 1 << 0
#define DOWNLOAD_NORMAL 1 << 1

#define CURR -2
#define CONSOLE 0
#define MOTD 1

typedef struct {
   char *conf_str;
   int key;
   int conf_val;
} ConfigTable;

typedef void (*GnapsterFreeFunc) (void *data);
typedef struct {
   int type;
   GnapsterFreeFunc func;
   size_t size;
} FreeTable;

typedef struct {
   unsigned short command;
   char *hook;
} HookTable;

typedef struct {
   int type;
} Type;

typedef struct {
   char *file;
   char *name;
   char *author;
} Theme;

typedef struct {
   char *name;
   char *data;
} ThemeHook;

typedef struct {
   char *line;
   char *lptr;
} FileEntry;

typedef struct {
   char *channel_name;
   int user;
   GtkWidget *text;
   GtkWidget *clist;
   GtkWidget *vbox;
} ChannelInfo;

#define OPENNAP_MASK (0xffff << 16)
#define CANCELS_MASK (0xff << 8)
#define CONNECTED_MASK (1 << 4)
#define CONNECTING_MASK (1 << 3)
#define SEARCHING_MASK (1 << 2)
#define BROWSING_MASK (1 << 1)
#define SAFE_EXIT_MASK (1 << 0)

typedef struct {
   /* AAAA AAAA AAAA AAAA BBBB BBBB 000C DEFG */
   /* A - opennap version
    * B - cancels
    * C - connected
    * D - connecting
    * E - searching
    * F - browsing
    * G - safe exit */
   guint32 state;

   int sock;
   int sock_input;
   int bind_sock;
   int bind_sock_input;
   int upload_input;
   int napigator_sock;
   int napigator_input;
   char *status;
   char *ip;
   int port;
   int nap_port;
   
   int reconnect_timeout;
} ConnInfo;

enum {
   NAME_FIX, LINE_SPEED_SWITCH, LINE_SPEED_CMP, LINE_SPEED,
   BITRATE_SWITCH, BITRATE_CMP, BITRATE, MAX_UPLOADS_SWITCH, MAX_UPLOADS,
   REJECT_UL, FIREWALL, MAX_PERUSER_UPLOADS_SWITCH, MAX_PERUSER_UPLOADS,
   AUTOQUERY, MAX_DOWNLOADS_SWITCH, MAX_DOWNLOADS,
   MAX_PERUSER_DOWNLOADS_SWITCH, MAX_PERUSER_DOWNLOADS, HIDE_USERS,
   LOGGING, HIDE_JOINPART, SOCKS_FIREWALL, SOCKS4, SOCKS5, TINT,
   TINT_RED, TINT_GREEN, TINT_BLUE, AUTOCANCEL, SEARCH_LOCAL, LIST_REFRESH,
   TERMINATION
};

typedef struct {
   char *user;
   char *pass;
   char *download_dir;
   char *upload_dir;
   int connection;
   int conf[TERMINATION + 1];
   int new_user;
   int sharing;
   unsigned long share_size;
   char *server;
   int server_port;
   char *default_font;
   char *theme;
} UserInfo;

void construct_path(GtkWidget *, Search *, Transfer *);
void check_resume(Transfer *);
Transfer *gnapster_match_real_download(char *, char *, int *);
int upload_accepted(char *, char *, char *);
char *gnapster_sv();
void gnapster_config_combo_widget(GtkWidget *, char *);;
void gnapster_config_toggle_widget(GtkWidget *, char *, ...);
int verify_file_shared(char *);
FILE *open_file(Transfer *, char *);
void *download_dup(Transfer *);
void *search_dup(Search *, int);
Transfer *gnapster_parse_download(char *, int);
Search *gnapster_parse_search(char *);
Search *gnapster_parse_resume(char *);
void gnapster_fill_dl_resume_path(Transfer *);
void gnapster_fill_resume_path(Search *);
Search *gnapster_parse_browse(char *, char **, int);
int gnapster_update_appbar();
void gnapster_update_status(char *, int);
void gnapster_highlight(GtkWidget *, GtkWidget *);
int user_lookup(char *);
int gnapster_get_pagenum();
int gnapster_clist_append(GtkWidget *, GtkWidget *, void *, ...);
void *gnapster_get_selection_data(GtkWidget *, int *);
void *gnapster_get_ctree_selection_data(GtkWidget *, int *);
void gnapster_get_styles(GtkWidget *);
void gnapster_config_set_int(int);
void gnapster_config_get_int();

#endif /* __GNAPSTER_H */
