#ifndef __NAPSTER_H
#define __NAPSTER_H

#include <stdio.h>
#include <glib.h>

#define NAPSTER_ERROR 0

#define NAPSTER_LOGIN 2
#define NAPSTER_LOGIN_ACK 3

#define NAPSTER_NEW_USER_LOGIN 6
#define NAPSTER_NEW_USER 7
#define NAPSTER_NEWUSER_SUCCESS 8
#define NAPSTER_NEWUSER_REG 9
#define NAPSTER_NEWUSER_INVALID_NICK 10

#define NAPSTER_SUBMIT 100
#define NAPSTER_REMOVE_SHARE 102

#define NAPSTER_SEARCH 200
#define NAPSTER_SEARCH_RESULT 201
#define NAPSTER_END_SEARCH 202

#define NAPSTER_GET 203
#define NAPSTER_DOWNLOAD_REPLY 204

#define NAPSTER_USER_MESSAGE 205

#define NAPSTER_DOWNLOAD_ERROR 206

#define NAPSTER_BROWSE_ERROR 210
#define NAPSTER_BROWSE 211
#define NAPSTER_BROWSE_RESULT 212
#define NAPSTER_END_BROWSE 213

#define NAPSTER_STATUS_MESSAGE 214

#define NAPSTER_RESUME_SEARCH 215
#define NAPSTER_RESUME_RESULT 216
#define NAPSTER_RESUME_END 217

#define NAPSTER_DOWNLOAD_START 218
#define NAPSTER_DOWNLOAD_COMPLETE 219
#define NAPSTER_UPLOAD_START 220
#define NAPSTER_UPLOAD_COMPLETE 221

#define NAPSTER_JOIN_CHAN 400
#define NAPSTER_PART_CHAN 401
#define NAPSTER_CHAN_MESSAGE 402
#define NAPSTER_CHAN_MESSAGE_IN 403

#define NAPSTER_SERVER_MESSAGE 404

#define NAPSTER_CHAN_JOIN 405
#define NAPSTER_CHAN_JOINUSER 406
#define NAPSTER_CHAN_PARTUSER 407
#define NAPSTER_CHAN_USERLIST 408
#define NAPSTER_CHAN_USERLIST_END 409
#define NAPSTER_CHAN_TOPIC 410

#define NAPSTER_CBAN_LIST 420
#define NAPSTER_CBAN_LIST_ENTRY 421
#define NAPSTER_CBAN 422
#define NAPSTER_CUNBAN 423
#define NAPSTER_CBAN_CLEAR 424

#define NAPSTER_ALT_GET 500
#define NAPSTER_ALT_DOWNLOAD_REPLY 501 

#define NAPSTER_WHOIS_REQUEST 603
#define NAPSTER_WHOIS_REPLY 604
#define NAPSTER_WHOIS_OFFLINE 605

#define NAPSTER_CHANGE_USERLEVEL 606

#define NAPSTER_UPLOAD_REQUEST 607
#define NAPSTER_ACCEPT_UPLOAD_REQUEST 608

#define NAPSTER_KILL_USER 610
#define NAPSTER_NUKE_USER 611

#define NAPSTER_BAN_USER 612
#define NAPSTER_UNBAN_USER 614

#define NAPSTER_BAN_LIST 615
#define NAPSTER_BAN_LIST_ENTRY 616

#define NAPSTER_REQUEST_PORTCHANGE 613

#define NAPSTER_CHAN_LIST 617
#define NAPSTER_CHAN_LIST_ENTRY 618

#define NAPSTER_SENDLIMIT 619
#define NAPSTER_SENDLIMIT_REACHED 620

#define NAPSTER_MESSAGE 621
#define NAPSTER_MUZZLE_USER 622
#define NAPSTER_UNMUZZLE_USER 623

#define NAPSTER_SET_LINESPEED 625

#define NAPSTER_PORT_ERROR 626

#define NAPSTER_ADMIN_MESSAGE 627
#define NAPSTER_GLOBAL_MESSAGE 628

#define NAPSTER_CHANGE_PORT 703

#define NAPSTER_PING 751
#define NAPSTER_PONG 752

#define NAPSTER_SERVER_VERSION 801

#define NAPSTER_ACTION 824

#define NAPSTER_ChannelInfoLIMIT 826

#define NAPSTER_KICK 829

#define NAPSTER_SUBMIT_DIRECTORY 870

/* Non-standard opennap commands */

#define NAPSTER_LINKS 10112

#define NAPSTER_SERVER_STATS 10115

#define NAPSTER_ChannelInfo_LEVEL 10201

#define NAPSTER_SET_USER_MODE 10203
#define NAPSTER_SET_ChannelInfo_OP 10204
#define NAPSTER_REMOVE_ChannelInfo_OP 10205

#define NAPSTER_ChannelInfo_OP_LIST 10206

#define NAPSTER_ChannelInfo_MODE 10209

#define NAPSTER_OPENNAP_BROWSE 10301
#define NAPSTER_OPENNAP_BROWSE_RESULT 10302

#define TRANSFER 0
#define N_SEARCH 1
#define N_SERVER 2
#define N_FILE 3
#define N_USER 4
#define N_CHAN_INFO 5
#define N_RESUME 6
#define W_PROP 7
#define HOOK 8
#define THEME 9
#define FILE_ENTRY 10

typedef struct {
   int type;
   
   char *trunc_file;
   char *file;
   char *user;
   
   int connection;
   unsigned short port;
   unsigned long int ip;
   
   int sock;
   int sock_input;
   
   int update_tag;
   
   int accept_timeout;
   int start_timeout;
   
   FILE *fdesc;
   
   char *fpath;
   
   char *header;
   
   unsigned long int size, temp_size;
   unsigned long int total;
   
   int calls;
   float kps;
   
   int timeout_calls;

   guint8 status;
   int timeout_tag;
   
   int ttype;
   int conn_type;
   
   char *id;
   int num;
   int kbps;
   int freq;
   char *resume_path;
} Transfer;

typedef struct {
   int type;
   
   char *file;
   char *trunc_file;
   
   char *id;
   unsigned long int size;
   unsigned short kbps;
   unsigned short freq;
   time_t num;
   
   char *user;
   unsigned long int ip;
   
   int connection;
   
   char *resume_path;
} Search;

typedef struct {
   int type;
   
   char *ip;
   int port;
   char *desc;
   char *network;
} Server;

typedef struct {
   int type;
   
   time_t mtime;
   char *filename;
} File;

typedef struct {
   int type;
   
   char *user;
   int shares;
   int conn;
} User;

typedef struct {
   int type;
   
   char *chan;
   int users;
   char *topic;
} Channel;

typedef struct {
   int type;
   
   char *file;
   char *checksum;
   int localsize;
   int remotesize;
} Resume;

int opennap_version(int, int);
int user_ignored(char *);
void napster_get_best_host();
void napster_connect(char *, int);
void napster_connect_user(Transfer *);
void napster_bind(int);
void napster_send_get(int, char *, char *, unsigned long int);
void napster_send_send(int, char *, char *, unsigned long int);
int napster_send(int, int, char *, ...);
int is_opennap_server(char *);
void napster_handle_data(int, char *);
int napster_conn_as_int(char *);
char *napster_conn_as_str(int);
int napster_user_uploads(char *);

#endif /* __NAPSTER_H */
